"""Test the pytest_houdini.tools module."""

# pytest-houdini
import pytest_houdini.tools

# Tests


def test_does_not_raise():
    """Test pytest_houdini.tools.does_not_raise().

    This test is pretty useless and mostly just here for test coverage.
    """
    with pytest_houdini.tools.does_not_raise():
        x = 1

    assert x == 1



"""Test the pytest_houdini.fixtures.other module."""

# Standard Library
import importlib

# pytest-houdini
import pytest_houdini.fixtures.other

importlib.reload(pytest_houdini.fixtures.other)

pytest_plugins = ["pytester"]


# Tests


def test_remove_abstract_methods(pytester, shared_datadir):
    """Test the 'remove_abstract_methods' fixture."""
    pytester.makepyfile(
        """
import abc

def test_remove_abstract_methods(remove_abstract_methods):

    class TestClass(metaclass=abc.ABCMeta):
        def foo(self):
            print("foo")

        @abc.abstractmethod
        def bar(self):
            pass

    remove_abstract_methods(TestClass)

    f = TestClass()
    f.foo()

"""
    )
    result = pytester.runpytest()

    result.assert_outcomes(passed=1)



"""Test the pytest_houdini.fixtures.shelf_tools module."""

# Standard Library
import importlib

# pytest-houdini
import pytest_houdini.fixtures.shelf_tools

importlib.reload(pytest_houdini.fixtures.shelf_tools)

pytest_plugins = ["pytester"]


# Tests


def test_exec_shelf_tool_script(pytester, shared_datadir):
    """Test the 'exec_shelf_tool_script' fixture."""
    shelf_test_file = shared_datadir / "test_shelf_files.shelf"

    pytester.makepyfile(
        f"""
import pytest

from pytest_houdini.fixtures.exceptions import MissingToolError
from pytest_houdini.tools import does_not_raise

import hou

@pytest.mark.parametrize(
    "tool_name, raiser",
    (
        ("_not_a_tool_", pytest.raises(MissingToolError)),
        ("pytest_houdini_test_tool", does_not_raise()),
    )
)
def test_exec_shelf_tool_script(exec_shelf_tool_script, tool_name, raiser):
    hou.shelves.loadFile("{shelf_test_file.as_posix()}")

    scriptargs = {{"result": False}}

    with raiser:
        exec_shelf_tool_script(tool_name, scriptargs)

        assert scriptargs["result"]
"""
    )
    result = pytester.runpytest()

    result.assert_outcomes(passed=2)



"""This module contains supporting tooling for testing in Houdini."""

# Future
from __future__ import annotations

# Standard Library
from contextlib import contextmanager
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from collections.abc import Generator


# Functions


@contextmanager
def does_not_raise() -> Generator[None, None, None]:
    """Dummy context manager for testing.

    You can use this to help parametrize tests which may or may not raise exceptions.

    Consider the below test for a function which divides two values. As it does not
    validate any of the values it will result in ZeroDivisionError if value2 is 0. We
    can use does_not_raise() in conjunction with pytest.raises(ZeroDivisionError) in order
    to test a bunch of values and handle any expected exceptions.

    def divider(value1, value2):
        return value1 / value2

    @pytest.mark.parametrize(
        "value, expected, tester",
        [
            (0, None, pytest.raises(ZeroDivisionError)),
            (3.0, 0.3333333333333333, does_not_raise()),
        ],
    )
    def test_divider(value, tester):

        with tester:
            result = divider(1, value)

            assert result == expected
    """
    yield



"""Test the pytest_houdini.fixtures.ui module."""

# Standard Library
import importlib

# pytest-houdini
import pytest_houdini.fixtures.ui

importlib.reload(pytest_houdini.fixtures.ui)

pytest_plugins = ["pytester"]


# Tests


def test_mock_hdefereval(pytester):
    """Test the 'mock_hdefereval' fixture."""
    pytester.makepyfile(
        """
import hou

def test_mock_hdefereval(mock_hdefereval):
    import hdefereval

    assert hdefereval == mock_hdefereval

"""
    )
    result = pytester.runpytest()

    result.assert_outcomes(passed=1)


def test_mock_hou_qt(pytester):
    """Test the 'mock_hou_qt' fixture."""
    pytester.makepyfile(
        """
import hou

# Ensure hou.qt does not exist before any manipulation of it.
def test_no_orig_qt():
    assert not hasattr(hou, "qt")


def test_mock_hou_qt(mock_hou_qt):
    assert hou.qt == mock_hou_qt


# Ensure hou.qt does not exist after any manipulation of it.
def test_no_post_qt():
    assert not hasattr(hou, "qt")

"""
    )
    result = pytester.runpytest()

    result.assert_outcomes(passed=3)


def test_mock_hou_ui(pytester):
    """Test the 'mock_hou_ui' fixture."""
    pytester.makepyfile(
        """
import hou

# Ensure hou.ui does not exist before any manipulation of it.
def test_no_orig_ui():
    assert not hasattr(hou, "ui")


def test_mock_hou_ui(mock_hou_ui):
    assert hou.ui == mock_hou_ui


# Ensure hou.ui does not exist after any manipulation of it.
def test_no_post_ui():
    assert not hasattr(hou, "ui")

"""
    )
    result = pytester.runpytest()

    result.assert_outcomes(passed=3)


def test_set_ui_available(pytester):
    """Test the 'set_ui_available' fixture."""
    pytester.makepyfile(
        """
import pytest

import hou

@pytest.mark.usefixtures("set_ui_available")
def test_set_ui_available():
    assert hou.isUIAvailable()

"""
    )
    result = pytester.runpytest()

    result.assert_outcomes(passed=1)



"""Testing fixtures for shelf tools."""

# Future
from __future__ import annotations

# Standard Library
from typing import Callable

# Third Party
import pytest

# pytest-houdini
from pytest_houdini.fixtures.exceptions import MissingToolError

# Houdini
import hou

# Fixtures


@pytest.fixture
def exec_shelf_tool_script() -> Callable:
    """Fixture to execute a shelf tool."""

    def _exec(tool_name: str, kwargs: dict) -> None:
        """Execute tool code inside a file.

        Args:
            tool_name: The name of the tool to execute.
            kwargs: The global 'kwargs' dict for the tool execution.

        Raises:
            MissingToolError: If a tool of the name cannot be found.
        """
        tool = hou.shelves.tool(tool_name)

        if tool is None:
            raise MissingToolError(tool_name)

        exec(tool.script(), {"kwargs": kwargs})

    return _exec



"""Test the pytest_houdini.fixtures.hip_file module."""

# Standard Library
import importlib
import shutil

# Third Party
import pytest

# pytest-houdini
import pytest_houdini.fixtures.hip_file

# Houdini
import hou

importlib.reload(pytest_houdini.fixtures.hip_file)

pytest_plugins = ["pytester"]


# Tests


def test_clear_hip_file(pytester):
    """Test the 'clear_hip_file' fixture."""
    pytester.makepyfile(
        """
import pytest

import hou

@pytest.mark.usefixtures("clear_hip_file")
def test_clear_hip_file(request):
    assert hou.node("/obj").children() == ()

    obj = hou.node("/obj")
    obj.createNode("null")

"""
    )

    # Create a node under /obj so there will be something there before the fixture
    # clears the hip file on setup.
    obj = hou.node("/obj")
    geo = obj.createNode("geo")

    # Run the test code.
    result = pytester.runpytest()

    result.assert_outcomes(passed=1)

    # Attempt to access the geom object we created.  This should fail as the node
    # was destroyed when the file was cleared.
    with pytest.raises(hou.ObjectWasDeleted):
        geo.path()

    # Ensure the null that was created during the test function was destroyed to
    # confirm that the file was cleared on cleanup.
    assert hou.node("/obj").children() == ()


@pytest.mark.parametrize("ext", [".hip", ".hiplc", ".hipnc", None])
def test_load_module_test_hip_file(pytester, ext, shared_datadir):
    """Test the 'load_module_test_hip_file' fixture."""
    data_dir = pytester.mkdir("data")

    expected_path = "untitled.hip"

    if ext is not None:
        hip_path = data_dir / f"test_load_module_test_hip_file{ext}"

        test_file = shared_datadir / "test_load_module_test_hip_file.hiplc"

        shutil.copy(test_file, hip_path)

        expected_path = hip_path.as_posix()

    pytester.makepyfile(
        f"""
import pytest

import hou

pytestmark = pytest.mark.usefixtures("load_module_test_hip_file")

def test_load_module_test_hip_file():
    assert hou.hipFile.path() == "{expected_path}"

"""
    )
    result = pytester.runpytest()

    if ext is None:
        result.assert_outcomes(errors=1)

    else:
        result.assert_outcomes(passed=1)


def test_set_test_frame(pytester):
    """Test the 'set_test_frame' fixture."""
    pytester.makepyfile(
        """
import hou

def test_set_test_frame(set_test_frame):
    set_test_frame(102)

    assert hou.frame() == 102

"""
    )

    hou.setFrame(101)

    assert hou.frame() == 101

    result = pytester.runpytest()

    assert hou.frame() == 101

    result.assert_outcomes(passed=1)



"""Fixtures to support mocking Mantra/SOHO related functionality."""

# Future
from __future__ import annotations

# Standard Library
import sys
from collections import namedtuple
from typing import TYPE_CHECKING, NamedTuple

# Third Party
import pytest

if TYPE_CHECKING:
    from pytest_mock import MockerFixture

# Fixtures


@pytest.fixture
def patch_soho(monkeypatch: pytest.MonkeyPatch, mocker: MockerFixture) -> NamedTuple:
    """Mock importing of mantra/soho related modules.

    Available mocked modules are available via their original names from the fixture provided named tuple.

    Available mocked modules:
        - IFDapi
        - IFDframe
        - IFDhooks
        - IFDsettings
        - mantra
        - soho

    >>> def test_soho_thing(patch_soho):
    ...     patch_soho.mantra.property.return_value = 3
    ...     # Test code
    """
    mock_api = mocker.MagicMock()
    mock_frame = mocker.MagicMock()
    mock_hooks = mocker.MagicMock()
    mock_mantra = mocker.MagicMock()
    mock_settings = mocker.MagicMock()
    mock_soho = mocker.MagicMock()

    monkeypatch.setitem(sys.modules, "IFDapi", mock_api)
    monkeypatch.setitem(sys.modules, "IFDframe", mock_frame)
    monkeypatch.setitem(sys.modules, "IFDhooks", mock_hooks)
    monkeypatch.setitem(sys.modules, "IFDsettings", mock_settings)
    monkeypatch.setitem(sys.modules, "mantra", mock_mantra)
    monkeypatch.setitem(sys.modules, "soho", mock_soho)

    MockSoho = namedtuple("MockSoho", "IFDapi IFDframe IFDhooks IFDsettings mantra soho")

    return MockSoho(mock_api, mock_frame, mock_hooks, mock_settings, mock_mantra, mock_soho)



"""Fixtures related to nodes."""

# Future
from __future__ import annotations

# Standard Library
import contextlib
from typing import TYPE_CHECKING, Callable

# Third Party
import pytest

# pytest-houdini
from pytest_houdini.fixtures.exceptions import (
    NoTestNodeError,
    TestNodeDoesNotContainSOPsError,
)

# Houdini
import hou

if TYPE_CHECKING:
    from collections.abc import Generator


# Non-Public Functions


def _find_matching_node(parent: hou.OpNode, request: pytest.FixtureRequest) -> hou.OpNode:
    """Try to find a matching child node based on a test request.

    Node search order is as follows:
    - Node matching the exact test name
    - Node matching the class name + test name (minus 'test_' prefix for function name)
    - Node matching the class name / test name (minus 'test_' prefix for function name)
    - Node matching the class name

    The class name can be either the raw name or entirely lowercase.

    Args:
        parent: The parent node to search under.
        request: A fixture request with which to find a node.

    Returns:
        A child node matching the request, if any.

    Raises:
         NoTestNodeError: Will be raised if no matching node could be found.
    """
    test_name = request.node.originalname

    # First try to find a node with the exact test name.
    names = [test_name]

    if request.cls is not None:
        cls_name = request.cls.__name__

        test_name = test_name[5:]

        # Look for a node with the class name + test name (minus test_ from function name)
        names.append(f"{cls_name}_{test_name}")
        names.append(f"{cls_name.lower()}_{test_name}")

        # Also support the test node being under a parent node based on the class name.
        names.append(f"{cls_name}/{test_name}")
        names.append(f"{cls_name.lower()}/{test_name}")

        # Finally try to find a node with the class name.
        names.append(cls_name)
        names.append(cls_name.lower())

    for name in names:
        node = parent.node(name)

        if node is not None:
            return node

    searched_paths = [parent.path() + "/" + name for name in names]

    raise NoTestNodeError(searched_paths)


# Fixtures


@pytest.fixture
def create_temp_node() -> Generator[Callable, None, None]:
    """Fixture to create a temporary node that will be destroyed on cleanup."""
    _created_nodes: list[hou.Node] = []

    def _create(
        parent: hou.Node, node_type_name: str, node_name: str | None = None, *, run_init_scripts: bool = True
    ) -> hou.Node:
        """Function to create a test node that will be destroyed on cleanup.

        Args:
            parent: The parent to create the test node under.
            node_type_name: The node type to create.
            node_name: Optional node name.
            run_init_scripts: Whether to run the node initialization scripts.

        Return:
            The created test node.
        """
        node = parent.createNode(node_type_name, node_name, run_init_scripts=run_init_scripts)

        _created_nodes.append(node)

        return node

    yield _create

    for created in _created_nodes:
        with contextlib.suppress(hou.ObjectWasDeleted):
            created.destroy()


@pytest.fixture
def obj_test_node(request: pytest.FixtureRequest) -> hou.OpNode:
    """Fixture to provide a node in /obj matching the test."""
    parent = hou.node("/obj")

    return _find_matching_node(parent, request)


@pytest.fixture
def obj_test_geo(obj_test_node: hou.OpNode) -> hou.Geometry:
    """Fixture to provide the read-only display node geometry of a node in /obj matching the test."""
    if obj_test_node.childTypeCategory() != hou.sopNodeTypeCategory():
        raise TestNodeDoesNotContainSOPsError(obj_test_node)

    return obj_test_node.displayNode().geometry()


@pytest.fixture
def obj_test_geo_copy(obj_test_geo: hou.Geometry) -> hou.Geometry:
    """Fixture to get a writable copy of the display node geometry of a node in /obj matching the test."""
    geo = hou.Geometry()

    geo.merge(obj_test_geo)

    return geo


@pytest.fixture
def out_test_node(request: pytest.FixtureRequest) -> hou.OpNode:
    """Fixture to provide a node in /out matching the test."""
    parent = hou.node("/out")

    return _find_matching_node(parent, request)



"""Exceptions raised by pytest-houdini."""

# Future
from __future__ import annotations

# Standard Library
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    import hou


# Exceptions


class MissingToolError(Exception):
    """Exception raised when a matching tool cannot be found.

    Args:
        tool_name: The missing tool name.
    """

    def __init__(self, tool_name: str) -> None:
        super().__init__(f"Could not find tool: {tool_name}")


class NoModuleTestFileError(Exception):
    """Exception raised when a module test file cannot be found.

    Args:
        test_file: The test file stem (no extension.)
        extensions: The extensions which were checked.
    """

    def __init__(self, test_file: str, extensions: tuple[str, ...]) -> None:
        msg = f"Could not find a valid test hip: {test_file}{{{','.join(extensions)}}}"

        super().__init__(msg)


class NoTestNodeError(Exception):
    """Exception raised when no test node could be found.

    Args:
        searched_paths: The test node paths which were searched.
    """

    def __init__(self, searched_paths: list[str]) -> None:
        super().__init__(f"Could not find any matching test nodes: {', '.join(searched_paths)}")


class TestNodeDoesNotContainSOPsError(Exception):
    """Exception raised when a test node does not contain SOP nodes.

    Args:
        node: The node which does not contain SOP nodes.
    """

    def __init__(self, node: hou.OpNode) -> None:
        super().__init__(f"{node.path()} does not contain SOP nodes.")



"""Initialize the pytest-houdini plugin."""

# Standard Library
from importlib.util import find_spec

# If Houdini is running then we can import our fixtures to expose.
if find_spec("hou"):
    from pytest_houdini.fixtures.hip_file import (
        clear_hip_file,
        load_module_test_hip_file,
        set_test_frame,
    )
    from pytest_houdini.fixtures.nodes import (
        create_temp_node,
        obj_test_geo,
        obj_test_geo_copy,
        obj_test_node,
        out_test_node,
    )
    from pytest_houdini.fixtures.other import remove_abstract_methods
    from pytest_houdini.fixtures.shelf_tools import exec_shelf_tool_script
    from pytest_houdini.fixtures.soho import patch_soho
    from pytest_houdini.fixtures.ui import (
        mock_hdefereval,
        mock_hou_qt,
        mock_hou_ui,
        set_ui_available,
    )



"""Fixtures related to hip files."""

# Future
from __future__ import annotations

# Standard Library
from typing import TYPE_CHECKING, Callable

# Third Party
import pytest

# pytest-houdini
from pytest_houdini.fixtures.exceptions import NoModuleTestFileError

# Houdini
import hou

if TYPE_CHECKING:
    from collections.abc import Generator

# Fixtures


@pytest.fixture
def clear_hip_file() -> Generator[None, None, None]:
    """Fixture to clear the current hip file before and after test running.

    >>> @pytest.mark.usefixtures("clear_hip_file")
    ... def test_something():
    ...     # Modify the scene state
    """
    hou.hipFile.clear(suppress_save_prompt=True)

    yield

    hou.hipFile.clear(suppress_save_prompt=True)


@pytest.fixture(scope="module")
def load_module_test_hip_file(
    request: pytest.FixtureRequest,
) -> Generator[None, None, None]:
    """Load a test hip file with the same name as the running module.

    Supports .hip, .hiplc, and .hipnc type files.

    The file must be under a data/ directory which is a sibling of the test file.

    The fixture will clear the hip file after the tests are completed.

    Example:
        Place the following line at the top of the test module::

        pytestmark = pytest.mark.usefixtures("load_module_test_hip_file")
    """
    test_file_path = request.path

    data_dir = test_file_path.parent / "data"

    hip_extensions = (".hip", ".hiplc", ".hipnc")

    for hip_ext in hip_extensions:
        # Look for a file with the test module's name and accompanying hip ext.
        test_file_name = test_file_path.with_suffix(hip_ext).name

        test_file_path = data_dir / test_file_name

        if test_file_path.is_file():
            break

    # We didn't find a matching file so raise an exception indicating no file could be found
    # and include the various paths we tried.
    else:
        raise NoModuleTestFileError((data_dir / test_file_path.stem).as_posix(), hip_extensions)

    hou.hipFile.load(test_file_path.as_posix(), suppress_save_prompt=True, ignore_load_warnings=True)

    yield

    hou.hipFile.clear(suppress_save_prompt=True)


@pytest.fixture
def set_test_frame() -> Generator[Callable, None, None]:
    """Fixed to set a frame for testing, restoring the previous one after completion.

    >>> def test_func(set_test_frame):
    ...    set_test_frame(1001)
    ...    # Do test things
    """
    current_frame = hou.frame()

    def _set_test_frame(frame: float) -> None:
        hou.setFrame(frame)

    yield _set_test_frame

    hou.setFrame(current_frame)



"""Other fixtures for testing."""

# Future
from __future__ import annotations

# Standard Library
from typing import Any, Callable

# Third Party
import pytest

# Fixtures


@pytest.fixture
def remove_abstract_methods(monkeypatch: pytest.MonkeyPatch) -> Callable:
    """Fixture to temporarily remove abstract methods from a class for testing purposes.

    Consider the following class definition with an abstract method and a concrete method which
    we want to test. Rather than creating a subclass for testing any non-abstract methods we
    can use the fixture to remove them during the test so that the object can be instantiated.

    class Foo(metaclass=abc.ABCMeta):

        @abc.abstractmethod
        def bar(self):
            pass

        def get_foo(self):
            return "foo"

    def test_get_foo(remove_abstract_methods):
        remove_abstract_methods(Foo)

        f = Foo()
        assert f.get_foo() == "foo"
    """

    def _remove_abstract(cls: Any) -> None:
        monkeypatch.setattr(cls, "__abstractmethods__", set())

    return _remove_abstract



"""Fixtures to support mocking Houdini UI related functionality."""

# Future
from __future__ import annotations

# Standard Library
import sys
from typing import TYPE_CHECKING

# Third Party
import pytest

# Houdini
import hou

if TYPE_CHECKING:
    from pytest_mock import MockerFixture


# Fixtures


@pytest.fixture
def mock_hdefereval(monkeypatch: pytest.MonkeyPatch, mocker: MockerFixture) -> MockerFixture:
    """Mock hdefereval which isn't available when running tests via Hython."""
    mocked_hdefereval = mocker.MagicMock()

    monkeypatch.setitem(sys.modules, "hdefereval", mocked_hdefereval)

    return mocked_hdefereval


@pytest.fixture
def mock_hou_qt(monkeypatch: pytest.MonkeyPatch, mocker: MockerFixture) -> MockerFixture:
    """Mock the hou.qt module which isn't available when running tests via Hython."""
    mock_qt = mocker.MagicMock()

    monkeypatch.setattr(hou, "qt", mock_qt, raising=False)

    return mock_qt


@pytest.fixture
def mock_hou_ui(monkeypatch: pytest.MonkeyPatch, mocker: MockerFixture) -> MockerFixture:
    """Mock the hou.ui module which isn't available when running tests via Hython."""
    mock_ui = mocker.MagicMock()

    monkeypatch.setattr(hou, "ui", mock_ui, raising=False)

    return mock_ui


@pytest.fixture
def set_ui_available(monkeypatch: pytest.MonkeyPatch) -> None:
    """Fixture to set hou.isUIAvailable() to return True."""
    monkeypatch.setattr(hou, "isUIAvailable", lambda: True)



"""Test the pytest_houdini.fixtures.nodes module."""

# Standard Library
import importlib

# pytest-houdini
import pytest_houdini.fixtures.nodes

# Houdini
import hou

importlib.reload(pytest_houdini.fixtures.nodes)

pytest_plugins = ["pytester"]


# Tests


def test_create_temp_node(pytester):
    """Test the 'create_temp_node' fixture."""
    pytester.makepyfile(
        """
import hou

# Test the basic fixture behavior.
def test_create_temp_node(create_temp_node):
    assert hou.node("/obj").children() == ()

    result = create_temp_node(hou.node("/obj"), "geo", "TEST_NODE")

    assert hou.node("/obj").children() == (result, )

    assert result.name() == "TEST_NODE"
    assert result.parm("viewportlod") is not None


# Test if we manually destroy the node instead of relying on automatic cleanup.
# Also tests run_init_scripts arg passing.
def test_create_temp_node_manual_destroy(create_temp_node):
    assert hou.node("/obj").children() == ()

    result = create_temp_node(hou.node("/obj"), "geo", "TEST_MANUAL_NODE", run_init_scripts=False)

    assert hou.node("/obj").children() == (result, )

    assert result.name() == "TEST_MANUAL_NODE"
    assert result.parm("viewportlod") is None

    result.destroy()


# Test using the fixture to create multiple nodes.
def test_create_temp_node_multiple(create_temp_node):
    assert hou.node("/obj").children() == ()

    result1 = create_temp_node(hou.node("/obj"), "geo", "TEST_NODE1")
    result2 = create_temp_node(hou.node("/obj"), "null", "TEST_NODE2")

    assert hou.node("/obj").children() == (result1, result2)


# Sentinel test to ensure that the test code is run inprocess and that things
# are being properly cleaned up.
def test_sentinel():
    hou.node("/obj").createNode("geo", "SENTINEL")

"""
    )
    result = pytester.runpytest()

    result.assert_outcomes(passed=4)

    assert hou.node("/obj").children() == (hou.node("/obj/SENTINEL"),)


def test_parametrized_node_names(pytester, shared_datadir):
    """Test to ensure node names are correct when using fixtures along with parametrized tests.

    The test nodes will be found according to the original name,
    test_obj_test_node_func, instead of test_obj_test_node_func[True|False].
    """
    test_hip = shared_datadir / "test_nodes.hiplc"

    pytester.makepyfile(
        f"""
import pytest

import hou

hou.hipFile.load("{test_hip.as_posix()}", ignore_load_warnings=True)

@pytest.mark.parametrize("extra_parm", (True, False))
def test_obj_test_node_func(obj_test_node, extra_parm, request):
    target_node = hou.node("/obj/test_obj_test_node_func")

    assert target_node is not None
    assert target_node == obj_test_node

"""
    )
    result = pytester.runpytest()

    result.assert_outcomes(passed=2)


def test_obj_test_node(pytester, shared_datadir):
    """Test the 'obj_test_node' fixture."""
    test_hip = shared_datadir / "test_nodes.hiplc"

    pytester.makepyfile(
        f"""
import hou

hou.hipFile.load("{test_hip.as_posix()}", ignore_load_warnings=True)


class TestNodes:
    def test_full_name(self, obj_test_node):
        target_node = hou.node("/obj/TestNodes_full_name")

        assert target_node is not None
        assert target_node == obj_test_node

    def test_nested_name(self, obj_test_node):
        target_node = hou.node("/obj/TestNodes/nested_name")

        assert target_node is not None
        assert target_node == obj_test_node

    def test_only_class_name(self, obj_test_node):
        target_node = hou.node("/obj/TestNodes")
        assert target_node == obj_test_node

class TestLowerNodes:
    def test_full_name(self, obj_test_node):
        target_node = hou.node("/obj/testlowernodes_full_name")

        assert target_node is not None
        assert target_node == obj_test_node

    def test_nested_name(self, obj_test_node):
        target_node = hou.node("/obj/testlowernodes/nested_name")

        assert target_node is not None
        assert target_node == obj_test_node

    def test_only_class_name(self, obj_test_node):
        target_node = hou.node("/obj/testlowernodes")
        assert target_node == obj_test_node

def test_obj_test_node_func(obj_test_node):
    target_node = hou.node("/obj/test_obj_test_node_func")

    assert target_node is not None
    assert target_node == obj_test_node


def test_no_matching(obj_test_node):
    pass

"""
    )
    result = pytester.runpytest()

    result.assert_outcomes(passed=7, errors=1)


def test_obj_test_geo(pytester, shared_datadir):
    """Test the 'obj_test_geo' fixture."""
    test_hip = shared_datadir / "test_nodes.hiplc"

    pytester.makepyfile(
        f"""
import hou

hou.hipFile.load("{test_hip.as_posix()}", ignore_load_warnings=True)

def test_obj_test_geo_missing(obj_test_geo):
    pass

def test_obj_test_geo_not_sop(obj_test_geo):
    pass

def test_obj_test_geo(obj_test_geo):
    target_node = hou.node('/obj/test_obj_test_geo/TARGET')

    assert obj_test_geo.sopNode() == target_node
    assert obj_test_geo.isReadOnly()

"""
    )
    result = pytester.runpytest()

    result.assert_outcomes(passed=1, errors=2)


def test_obj_test_geo_copy(pytester, shared_datadir):
    """Test the 'obj_test_geo_copy' fixture."""
    test_hip = shared_datadir / "test_nodes.hiplc"

    pytester.makepyfile(
        f"""
import hou

hou.hipFile.load("{test_hip.as_posix()}", ignore_load_warnings=True)

def test_obj_test_geo_copy(obj_test_geo_copy):
    target_node = hou.node('/obj/test_obj_test_geo_copy/TARGET')

    assert obj_test_geo_copy.sopNode() is None
    assert not obj_test_geo_copy.isReadOnly()
    assert obj_test_geo_copy.intrinsicValue("pointcount") == target_node.geometry().intrinsicValue("pointcount")

"""
    )
    result = pytester.runpytest()

    result.assert_outcomes(passed=1)


def test_out_test_node(pytester, shared_datadir):
    """Test the 'out_test_node' fixture."""
    test_hip = shared_datadir / "test_nodes.hiplc"

    pytester.makepyfile(
        f"""
import hou

hou.hipFile.load("{test_hip.as_posix()}", ignore_load_warnings=True)

def test_out_test_node(out_test_node):
    target_node = hou.node("/out/test_out_test_node")

    assert target_node is not None
    assert target_node == out_test_node

"""
    )
    result = pytester.runpytest()

    result.assert_outcomes(passed=1)



"""setup.py for pytest_houdini."""
# Standard Library
from pathlib import Path

# Third Party
from setuptools import find_packages, setup

# Package meta-data.
NAME = "pytest-houdini"
DESCRIPTION = "pytest plugin for testing code in Houdini."
URL = "https://github.com/captainhammy/pytest-houdini"
AUTHOR = "Graham Thompson"
AUTHOR_EMAIL = "captainhammy@gmail.com"
REQUIRES_PYTHON = ">=3.9.0"

this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text()

setup(
    name=NAME,
    use_scm_version=True,
    description=DESCRIPTION,
    long_description=long_description,
    long_description_content_type="text/markdown",
    author=AUTHOR,
    author_email=AUTHOR_EMAIL,
    python_requires=REQUIRES_PYTHON,
    url=URL,
    package_dir={"": "src"},
    packages=find_packages(where="src", exclude=("tests",)),
    install_requires=[
        "pytest",
        "pytest-datadir",
        "pytest-mock",
    ],
    extras_require={
        "test": [
            "coverage",
            "pytest-cov",
            "tox",
        ]
    },
    entry_points={
        "pytest11": ["pytest-houdini = pytest_houdini.plugin"],
    },
    include_package_data=True,
    license="MIT",
    classifiers=[
        "Framework :: Pytest",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3 :: Only",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3.12",
        "Topic :: Software Development :: Testing",
    ],
    project_urls={"Documentation": "https://pytest-houdini.readthedocs.io/"},
)



"""Test the pytest_houdini.fixtures.soho module."""

# Standard Library
import importlib

# pytest-houdini
import pytest_houdini.fixtures.soho

importlib.reload(pytest_houdini.fixtures.soho)

pytest_plugins = ["pytester"]


# Tests


def test_patch_soho(pytester):
    """Test the 'patch_soho' fixture."""
    pytester.makepyfile(
        """
import hou

def test_patch_soho(patch_soho):
    import IFDapi
    assert IFDapi == patch_soho.IFDapi

    import IFDhooks
    assert IFDhooks == patch_soho.IFDhooks

    import IFDsettings
    assert IFDsettings == patch_soho.IFDsettings

    import mantra
    assert mantra == patch_soho.mantra

    import soho
    assert soho == patch_soho.soho

"""
    )
    result = pytester.runpytest()

    result.assert_outcomes(passed=1)
